unit Animation;

interface

uses
  SysUtils, Classes, Graphics, ExtCtrls;

type
  TAnimationFrame = class(TObject)
  private
    FDeltaY: Integer;
    FDeltaX: Integer;
    FName: string;
    FImage: TBitmap;
    FTimeCount: Word;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure SaveToStream(Stream: TStream); virtual;
    procedure LoadFromStream(Stream: TStream); virtual;
    property DeltaX: Integer read FDeltaX write FDeltaX;
    property DeltaY: Integer read FDeltaY write FDeltaY;
    property Image: TBitmap read FImage write FImage;
    property Name: string read FName write FName;
    property TimeCount: Word read FTimeCount write FTimeCount;
  end;

  TCustomAnimation = class(TObject)
  protected
    FramesList: TList;
  public
    constructor Create; virtual;
    destructor Destroy; override;
    procedure LoadFromStream(Stream: TStream); virtual;
    procedure SaveToStream(Stream: TStream); virtual;
    procedure LoadFromFile(FileName: string);
    procedure SaveToFile(FileName: string);
  end;

  TChangeFrameEvent = procedure(Sender: TCustomAnimation;
    Frame: TAnimationFrame)of object;

  TAnimation = class(TCustomAnimation)
  private
    FOnFrameChange: TChangeFrameEvent;
    FrameIndex: Integer;
    FromIndex: Integer;
    ToIndex: Integer;
    FOnEndPlay: TChangeFrameEvent;
    FOnBedinPlay: TChangeFrameEvent;
  protected
    Timer: TTimer;
    procedure TimeProc(Sender: TObject); virtual;
  public
    constructor Create; override;
    destructor Destroy; override;
    property OnFrameChange: TChangeFrameEvent read FOnFrameChange write FOnFrameChange;
    property OnBedinPlay: TChangeFrameEvent read FOnBedinPlay write FOnBedinPlay;
    property OnEndPlay: TChangeFrameEvent read FOnEndPlay write FOnEndPlay;
    function FramesCount: Integer;
    function GetFrameIndex(FrameName: string): Integer; virtual;
    procedure Play(PlayFrom, PlayTo: Integer); overload; virtual;
    procedure Play(PlayFrom, PlayTo: string); overload; virtual;
    procedure Stop;
  end;

  TAnimationBuild = class(TAnimation)
  private
    function GetFrame(Index: Integer): TAnimationFrame;
    procedure SetFrame(Index: Integer; const Value: TAnimationFrame);
  public
    function AddFrame: TAnimationFrame; virtual;
    procedure DelFrame(Index: Integer); overload; virtual;
    procedure DelFrame(Name: string); overload; virtual;
    procedure MoveFrame(FirstIndex, NewIndex: Integer);
    function FrameExists(Index: Integer): Boolean; overload; virtual;
    function FrameExists(Name: string): Boolean; overload; virtual;
    property Frame[Index: Integer]: TAnimationFrame read GetFrame write SetFrame;
  published
    property OnFrameChange;
  end;

implementation

{ TAnimationFrame }

constructor TAnimationFrame.Create;
begin
  FImage:=TBitmap.Create;
  FDeltaX:=0;
  FDeltaY:=0;
end;

destructor TAnimationFrame.Destroy;
begin
  FImage.Free;
  inherited;
end;

procedure TAnimationFrame.LoadFromStream(Stream: TStream);
var
  NameSz: Integer;
begin
  NameSz:=0;
  Stream.Read(NameSz, SizeOf(NameSz));
  SetLength(FName, NameSz);
  Stream.Read(Pointer(FName)^, NameSz);
  Stream.Read(FTimeCount, SizeOf(FTimeCount));
  Stream.Read(FDeltaX, SizeOf(FDeltaX));
  Stream.Read(FDeltaY, SizeOf(FDeltaY));
  FImage.LoadFromStream(Stream);
end;

procedure TAnimationFrame.SaveToStream(Stream: TStream);
var
  NameSz: Integer;
begin
  NameSz:=Length(FName);
  Stream.Write(NameSz, SizeOf(NameSz));
  Stream.Write(Pointer(FName)^, NameSz);
  Stream.Write(FTimeCount, SizeOf(FTimeCount));
  Stream.Write(FDeltaX, SizeOf(FDeltaX));
  Stream.Write(FDeltaY, SizeOf(FDeltaY));
  FImage.SaveToStream(Stream);
end;

{ TCustomAnimation }

constructor TCustomAnimation.Create;
begin
  FramesList:=TList.Create;
end;

destructor TCustomAnimation.Destroy;
begin
  FramesList.Free;
  inherited;
end;

procedure TCustomAnimation.LoadFromFile(FileName: string);
var
  FileStream: TFileStream;
begin
  FileStream:=TFileStream.Create(FileName, fmOpenRead);
  LoadFromStream(FileStream);
  FileStream.Free;
end;

procedure TCustomAnimation.LoadFromStream(Stream: TStream);
var
  Cnt: Integer;
  I: Integer;
  NewFrame: TAnimationFrame;
begin
  FramesList.Clear;
  Stream.Read(Cnt, SizeOf(Cnt));
  for I:=0 to Cnt-1 do
  begin
    NewFrame:=TAnimationFrame.Create;
    NewFrame.LoadFromStream(Stream);
    FramesList.Add(NewFrame);
  end;
end;

procedure TCustomAnimation.SaveToFile(FileName: string);
var
  FileStream: TFileStream;
begin
  FileStream:=TFileStream.Create(FileName, fmCreate);
  SaveToStream(FileStream);
  FileStream.Free;
end;

procedure TCustomAnimation.SaveToStream(Stream: TStream);
var
  Cnt: Integer;
  I: Integer;
  SavedFrame: TAnimationFrame;
begin
  Cnt:=FramesList.Count;
  Stream.Write(Cnt, SizeOf(Cnt));
  for I:=0 to Cnt-1 do
  begin
    SavedFrame:=FramesList[I];
    SavedFrame.SaveToStream(Stream);
  end;
end;

{ TAnimation }

function TAnimation.FramesCount: Integer;
begin
  Result:=FramesList.Count;
end;

procedure TAnimation.Play(PlayFrom, PlayTo: string);
var
  X1, X2: Integer;
begin
  X1:=GetFrameIndex(PlayFrom);
  X2:=GetFrameIndex(PlayTo);
  Play(X1, X2);
end;

procedure TAnimation.Play(PlayFrom, PlayTo: Integer);
begin
  if not ((PlayFrom>=0)and(PlayFrom<=FramesCount-1)and
    (PlayTo>=0)and(PlayTo<=FramesCount-1)) then Exit;
  FromIndex:=PlayFrom;
  ToIndex:=PlayTo;
  FrameIndex:=FromIndex;
  if FromIndex<>ToIndex then
  begin
    Timer.Enabled:=true;
    Timer.OnTimer(Timer);
  end;
end;

procedure TAnimation.Stop;
begin
  FrameIndex:=0;
  FromIndex:=0;
  ToIndex:=0;
  Timer.Enabled:=false;
end;

function TAnimation.GetFrameIndex(FrameName: string): Integer;
var
  I: Integer;
  Frame: TAnimationFrame;
begin
  Result:=-1;
  for I:=0 to FramesList.Count-1 do
  begin
    Frame:=FramesList[I];
    if UpperCase(Frame.Name)=UpperCase(FrameName)then
    begin
      Result:=I;
      Exit;
    end;
  end;
end;

constructor TAnimation.Create;
begin
  inherited;
  FrameIndex:=0;
  Timer:=TTimer.Create(nil);
  Timer.OnTimer:=TimeProc;
  Timer.Enabled:=false;
end;

destructor TAnimation.Destroy;
begin
  Timer.Free;
  inherited;
end;

procedure TAnimation.TimeProc(Sender: TObject);
var
  Frame: TAnimationFrame;
begin
  if ((FromIndex<ToIndex)and(FrameIndex>ToIndex))or
    ((FromIndex>ToIndex)and(FrameIndex<ToIndex))then
  begin
    Frame:=FramesList[ToIndex];
    Stop;
    if Assigned(FOnEndPlay)then
      FOnEndPlay(Self, Frame);
    Exit;
  end;
  Frame:=FramesList[FrameIndex];
  if FrameIndex=FromIndex then
    if Assigned(FOnBedinPlay)then
      FOnBedinPlay(Self, Frame);
  if Assigned(FOnFrameChange)then
    FOnFrameChange(Self, Frame);

  Timer.Interval:=Frame.TimeCount;

  if FromIndex<ToIndex then
    FrameIndex:=FrameIndex+1;
  if FromIndex>ToIndex then
    FrameIndex:=FrameIndex-1;
end;

{ TAnimationBuild }

function TAnimationBuild.AddFrame: TAnimationFrame;
begin
  Result:=TAnimationFrame.Create;
  FramesList.Add(Result);
end;

procedure TAnimationBuild.DelFrame(Index: Integer);
begin
  if FrameExists(Index)then
    FramesList.Delete(Index);
end;

function TAnimationBuild.FrameExists(Index: Integer): Boolean;
begin
  Result:=(Index>=0)and(Index<=FramesCount-1);
end;

function TAnimationBuild.FrameExists(Name: string): Boolean;
var
  Index: Integer;
begin
  Index:=GetFrameIndex(Name);
  Result:=FrameExists(Index);
end;

procedure TAnimationBuild.DelFrame(Name: string);
begin
  if FrameExists(Name)then
    FramesList.Delete(GetFrameIndex(Name));
end;

procedure TAnimationBuild.MoveFrame(FirstIndex, NewIndex: Integer);
begin
  if FrameExists(FirstIndex)and FrameExists(NewIndex)then
    FramesList.Move(FirstIndex, NewIndex);
end;

function TAnimationBuild.GetFrame(Index: Integer): TAnimationFrame;
begin
  Result:=nil;
  if FrameExists(Index)then
    Result:=FramesList[Index];
end;

procedure TAnimationBuild.SetFrame(Index: Integer;
  const Value: TAnimationFrame);
begin
  if FrameExists(Index)then
    FramesList[Index]:=Value;
end;

end.
